% COMP2TENS returns the 3x3 tensor given its five components of on a basis
% of the 3x3 traceless symmetric tensors. 
%
% usage: [T] = Comp2Tens(e)
%
% where size(e) = [...,5] and size(T) = [...,3,3].  

function [T] = Comp2Tens(e)
sizeE = size(e);
if (sizeE(end) ~= 5) 
    error('The row dimension of the input argument is not five.');
end
% Define the basis of the 3x3 traceless symmetric tensors.  Define them as
% 9x9 vectors so that we can use a matrix product to compute the linear
% combination of the basis elements.
B = zeros(5,3,3);
B(1,1,1) = -1/sqrt(6);
B(1,2,2) = -1/sqrt(6);
B(1,3,3) = 2/sqrt(6);
B(2,1,1) = 1/sqrt(2);
B(2,2,2) = -1/sqrt(2);
B(3,1,2) = 1/sqrt(2);
B(3,2,1) = 1/sqrt(2);
B(4,1,3) = 1/sqrt(2);
B(4,3,1) = 1/sqrt(2);
B(5,2,3) = 1/sqrt(2);
B(5,3,2) = 1/sqrt(2);
B = reshape(B,5,9);
% Compute the tensor T as a nine component vector per each grid point (the
% grid points are cast as a one dimensional vector as well).
T = reshape(e,[numel(e)/5,5])*B;
% Make the dimensions of T compatible with those of the input vector e.
if (sizeE(1) > 1)
    T = reshape(T,[sizeE(1:end-1),3,3]);
else
    T = reshape(T,3,3);
end