% TENS2EIG given a nxn tensor T returns its eigenvalues and eigenvectors.
%
% usage: [eigenvals,eigenvects] = Tens2Eig(T)
%
% where size(T) = [...,n,n] while size(eigenvals) = [...,n] and
% size(eigenvecs) = [...,n,n].

function [vecs,vals] = Tens2Eig(T)
if (ndims(T)<2) 
    error('The argument must be a tensor or an array of tensors.');
end
sizeT = size(T);
if (sizeT(end-1) ~= sizeT(end)) 
    error('The tensor is not square.');
end
nDim = sizeT(end);
nGrid = numel(T)/nDim^2;
% Define the work arrays.
vals = zeros(nGrid,nDim);             % Eigenvalues
vecs = zeros(nGrid,nDim,nDim);        % Eigenvectors
T_tmp = reshape(T,[nGrid,nDim,nDim]); % Input tensor.
% Compute the eigenvalues and eigenvectors for each grid point in a loop.
for k=1:nGrid
    d1(:,:) = T_tmp(k,:,:);
    [vecs(k,:,:),dummy] = eig(d1);
    vals(k,:) = diag(dummy);
end
% Reshape the eigenvalues and eigenvectors arrays for to be in a format
% compatible with T.
if (ndims(T)>2) 
    vals = reshape(vals,sizeT(1:end-1));
    vecs = reshape(vecs,sizeT);
else
    vals = squeeze(vals);
    vecs = squeeze(vecs);
end